/* parseshf.c
   Program for parsing and converting shf.dtd compliant 
   XML files to raw memory chunks.
   Requires Expat (commonly packaged as xmltok-dev)
   Copyright (C) 2003 Linus Walleij


   This file is part of the shftool package.

   Shftool is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   You should have received a copy of the GNU General Public License
   along with Shftool; see the file COPYING.  If not, write to
   the Free Software Foundation, 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA. 

*/

#include "common.h"

// Read this from environment $COLUMNS
#define COLUMNS 80

// Has the dump begun?
bool DumpStarted = false;
bool DataThroughput = false;

// Specifics about this dump
u_int64_t blockcount;
u_int64_t readblocks;

// Specifics about each block
u_int64_t address;
u_int64_t wordlen;
u_int64_t blocklen;
u_int64_t readwords;
u_int8_t checksum[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

// Data word scanning
int currentnybble;
u_int8_t currenthinybble;
u_int8_t currentlonybble;
u_int8_t currentbyte;
int currentwordbyte;
u_int8_t *currentword;

// GUI related
int usedCols;

static void checksumFromStr(const char *str) {
  int i, j, k;
  u_int8_t hinybble;
  u_int8_t lonybble;

  j = 0;
  k = 0;
  for (i = 0; i < strlen(str); i++) {
    // Remove illegal characters
    if (filterHex(str[i]) != '\0') {
      u_int8_t b = hexToNybble(str[i]);
      if (j == 0) {
	hinybble = b << 4;
	j ++;
      } else {
	lonybble = b;
	checksum[k] = hinybble | lonybble;
	if (k < 20) {
	  k++;
	} else {
	  OutString("%s", "Invalid checksum size. Aborting.");
	  exit(1);
	}
	j = 0;
      }
    }
  }
  if (k != 20) {
    OutString("%s", "Invalid checksum size. Aborting.");
    exit(1);
  }
}


/****************************************************************
 * This delimiter marks the beginning of exported "non-static"
 * functions.
 ****************************************************************/

// Start handler for XML elements
void
startElementHandler(void *data, const XML_Char *el, const XML_Char **attr) 
{
  int i;

  if (!strcmp(el,"dump")) {
    DumpStarted = true;
    readblocks = 0;
    OutString("Dump:\n");
    for (i = 0; attr[i]; i += 2) {
      if (!strcmp(attr[i], "name"))
	OutString(" Dump name: %s\n", attr[i+1]);
      else if (!strcmp(attr[i], "blocks")) {
	blockcount = hexStrToULint(attr[i+1]);
	OutString(" Number of blocks: %llx\n", blockcount);
      }
      else {
	OutString(" Invalid attribute: %s, value: %s\n", attr[i], attr[i+1]);
      }
    }
  }
  else if (!strcmp(el,"block")) {
    readwords = 0;
    readblocks ++;
    initialize_sha1();
    OutString("Block %llx:\n", readblocks);
    for (i = 0; attr[i]; i += 2) {
      if (!strcmp(attr[i], "name"))
	OutString(" Block name: %s\n", attr[i+1]);
      else if (!strcmp(attr[i], "address")) {
	address = hexStrToULint(attr[i+1]);
	OutString(" Address: %016llx\n", address);
      }
      else if (!strcmp(attr[i], "word_size")) {
	wordlen = hexStrToULint(attr[i+1]);
	currentword = malloc(wordlen);
	OutString(" Word size (in bytes): %llx\n", wordlen);
      }
      else if (!strcmp(attr[i], "length")) {
	blocklen = hexStrToULint(attr[i+1]);
	OutString(" Length (in words): %llx\n", blocklen);
      }
      else if (!strcmp(attr[i], "checksum")) {
	int j;
	checksumFromStr(attr[i+1]);
	OutString(" SHA-1 checksum:");
	dumpHexArray(checksum, 20);
	OutString("\n");
      }
      else {
	OutString(" Invalid attribute: %s, value: %s\n", attr[i], attr[i+1]);
      }
    }
  }
  else if (!strcmp(el,"data")) {
    // Start handling data
    DataThroughput = true;
    currentnybble = 0;
    currentwordbyte = 0;
    usedCols = 0;
    OutString("Data:\n");
    for (i = 0; attr[i]; i += 2) {
      OutString(" Invalid attribute: %s, value: %s\n", attr[i], attr[i+1]);
    }
  }
  else {
    OutString("Invalid element: %s", el);
    for (i = 0; attr[i]; i += 2) {
      OutString(" Invalid attribute: %, value: %s\n", attr[i], attr[i+1]);
    }
  }
}


// End handler for XML elements
void 
endElementHandler(void *data, const XML_Char *el)
{
  if (!strcmp(el,"dump")) {
    DumpStarted = false;
    if (readblocks != blockcount) {
      OutString(" Incorrect number of blocks! Expected %llx got %llx\n", blockcount, readblocks);
    }
  } else if (!strcmp(el,"block")) {
    // Block ended...
    u_int8_t *calcsum = retrieve_sha1(readwords * wordlen * 8);
    if (readwords != blocklen) {
      OutString(" Incorrect number of words! Expected %llx got %llx\n", blocklen, readwords);
    }
    OutString("\n Calculated SHA-1 checksum:");
    dumpHexArray(calcsum, 20);
    OutString("\n");
    OutString(" Please match:             ");
    dumpHexArray(calcsum, 20);
    OutString("\n");
    free(currentword);
    free(calcsum);
  } else if (!strcmp(el,"data")) {
    DataThroughput = false;
  } else {
    OutString(" End of invalid element: %s", el);
  }
}


// The raw text falls through to here
void 
textHandler(void *data, const XML_Char *text, int length)
{
  if (!DumpStarted) {
    // Do nothing of the Dump hasn't started
  } else if (DataThroughput) {
    // Wow dump data!
    int i;
    char plotchar;

    for (i=0;i<length;i++) {
      plotchar = (char) text[i];
      // Silently ignore anything else than [0-9][a-f][A-F]
      if (filterHex(plotchar) != '\0') {
	if (currentnybble == 0) {
	  currenthinybble = hexToNybble(plotchar) << 4;
	  currentnybble ++;
	} else {
	  u_int8_t currentbyte;
	  currentlonybble = hexToNybble(plotchar);
	  currentbyte = currenthinybble | currentlonybble;
	  currentword[currentwordbyte] = currentbyte;
	  currentwordbyte ++;
	  add_byte_to_sha1(currentbyte);
	  if (currentwordbyte == wordlen) {
	    readwords ++;
	    if (usedCols + (wordlen*2) + 1 > COLUMNS) {
	      OutString("\n");
	      OutString(" ");
	      usedCols = wordlen * 2 + 2;
	    } else {
	      OutString(" ");
	      usedCols += wordlen*2 + 1;
	    }
	    dumpHexArray(currentword, wordlen);
	    currentwordbyte = 0;
	  }
	  currentnybble = 0;
	}
      }
    }
  }
}

// All raw entities are handled here
void 
defaultHandler(void *userData, const XML_Char *s, int length)
{
  register int i;
  static char temp_string_1[15]; /* How long can an escape code be? */
  char *temp_string_2;

  if (s[0]=='&' && s[length-1]==';') {
    // Output only valid escape codes for [0-9][a-f][A-F], ignore anything else.
    // &0x30; &0x31; ...? &48; &49; ...?
  }
}

/* scans for tool-specific options and sets variables correspondingly */
void
ScanToolOptions(const char **argv, int arg) {
  /* No tool specific options */
}

/* Print any tool specific help */
void
PrintToolHelp(void) {
  /* Implement a generic charset translator */
  printf("         -foo do bar\n");
}

/* End exported functions */
