/* cmdline.c
   Program for general command line interface
   Requires Expat (commonly packaged as xmltok-dev)
   Copyright (C) 2003 Linus Walleij

   This file is part of the shftool package.

   Shftool is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   You should have received a copy of the GNU General Public License
   along with Shftool; see the file COPYING.  If not, write to
   the Free Software Foundation, 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA. 

*/

#include "common.h"

/* Extra includes used by this file only */
#include <ctype.h>
#include <stdarg.h>
#include <dirent.h>
#include <sys/stat.h>


/* Output file */
static FILE* OutFile;
/* Buffer for output strings (linebreaking etc) */
static char Buff[BUFFSIZE];
static bool OldLineBreakMode;
static bool OldPreserveWhitespace;


void OutString(char *fmt, ...)
{
  va_list ap;
 
  va_start(ap, fmt);
  vfprintf(OutFile, fmt, ap);
  va_end(ap);
}

void OutChar(char c)
{
  fputc(c, OutFile);
}


/************************************************************************
 * Main function call to XML translators
 ************************************************************************/
int
main(const int argc, const char **argv) {
  // Variables
  XML_Parser parser;
  bool done;
  int len;
  int arg;
  // Signifies if the program should write to stdout instead of a file
  bool Stdout = false;
  // Dump to output
  bool DumpMode = false;
  char cDir[STR_MAXLEN] = "";
  char cInFile[STR_MAXLEN] = "";
  char cOutFile[STR_MAXLEN] = "";
  static char cProgramName[]= PACKAGE "-" VERSION;
  FILE* InFile;

  if (argc<2) {
    printf("Usage: %s [-options] <file.shf>\n", argv[0]);
    printf("Type %s -help for all options.\n", argv[0]);
    exit(0);
  }

  for (arg=1; arg<argc; arg++) {
    if (*argv[arg] == '-') {
      // -? or -help: show the command line help page
      if (!strcmp(argv[arg],"-?") || !strcmp(argv[arg],"-help") || !strcmp(argv[arg],"--help")) {
	printf("Usage: %s [-options] <file[.shf]>\n\n", argv[0]);
	printf("Options: -             = Write to stdout\n");
	// Print any tool-specific help text
	PrintToolHelp();
	exit(0);
	// non-interactive
      }
      if (!strcmp(argv[arg],"-"))
	Stdout = true;
      // Checks for tool-specific options (implemented in all subtools)
      ScanToolOptions(argv, arg);
    }
  }

  // See that the source is OK
  strcpy(cInFile, argv[argc-1]);
  if (strcmp(cInFile+(strlen(cInFile)-4), ".shf"))
    strcpy(cInFile+strlen(cInFile), ".shf");
  
  if(!Stdout) {
    strcpy(cOutFile, cInFile);
    strcpy(cOutFile+strlen(cOutFile)-3,argv[0]+strlen(argv[0])-3);
    fprintf(stderr, "Input file: %s\n", cInFile);
    fprintf(stderr, "Output file: %s\n", cOutFile);
  }

  // Scan for the file or just assume it is here and open it 
  InFile = fopen(cInFile, "r");
  if (InFile == NULL) {
    fprintf(stderr, "Could not open input file <%s!>\n", cInFile);
    exit(-1);
  }

  if(!Stdout) {
    OutFile = fopen(cOutFile, "w");
    if (OutFile == NULL) {
      fprintf(stderr, "Could not create output file <%s>!\n", cOutFile);
      exit(-1);
    }
  } else {
    OutFile = stdout;
  }

  parser = XML_ParserCreate(NULL);
    
  if (! parser) {
    fprintf(stderr, "Could not create an XML parser instance\n");
    exit(-1);
  }

  // Create an XML parser
  XML_SetElementHandler(parser, startElementHandler, endElementHandler);
  XML_SetCharacterDataHandler(parser, textHandler);
  XML_SetDefaultHandler(parser, defaultHandler);

  if (fseek(InFile, 0, SEEK_SET)) {
    fprintf(stderr, "Could not read from input file <%s>!\n", cInFile);
    exit(-1);
  }
  done = false;
  while (!done) {
    len = fread(Buff, 1, BUFFSIZE, InFile);
    if (ferror(InFile)) {
      fprintf(stderr, "Could not read from input file <%s>!\n", cInFile);
      exit(-1);
    }
    done = feof(InFile);
    
    if (! XML_Parse(parser, Buff, len, done)) {
      fprintf(stderr, "Parse error at line %d:\n%s\n",
	      XML_GetCurrentLineNumber(parser),
	      XML_ErrorString(XML_GetErrorCode(parser)));
      XML_ParserFree(parser);
      exit(-1);
    }
  }
  XML_ParserFree(parser);

  fclose(InFile);
  OutChar('\n');
  if (!Stdout)
    fclose(OutFile);
  exit(0);
}
